#version 120

uniform sampler2D       Texture;
uniform sampler2D       Scene;
uniform sampler2D       Dirt;
uniform vec4            Param1; // (SunPosX, SunPosY, 1.0/fbo.width, 0.5/FlareTexture.width)
uniform vec4            Param2; // (Radius, Stride, Bright, Scale)
uniform vec4            tint; // (Radius, Stride, Bright, Scale)
uniform float			screenRatio; // GLFrame.getHeight() / GLFrame.getWidth()

const float exposure = 0.3;
const float decay = 0.3;
const float density = 0.95;
const float densityInv = 0.93333;
const float weight = 0.065;
const float weight2 = 0.225;
const float weight3 = 0.095;
const float weight4 = 0.009;


const float Sexposure = 0.27;
const float Sdecay = 0.097;
const float Sdensity = 1.0;
const float Sweight = 0.025;
/*
float Density = 3;
float Exposure = -.01;
float Weight = .25;
float Weight2 = .125;
float Decay = .5;
*/

//uniform vec2 lightPositionOnScreen;
uniform sampler2D firstPass;
const int NUM_SAMPLES = 350 ;
	
	vec4 DoLenseFlare(vec2 ScreenLightPosition,vec2 texCoord,bool fwd, float weight, float xTC, float dist) {

	
	    // Calculate vector from pixel to light source in screen space.
	    vec2 deltaTexCoord = (texCoord - ScreenLightPosition.xy);// Divide by number of samples and scale by control factor.
	    
	     
	    deltaTexCoord *= (dist) *  densityInv*2;
	    
	    //deltaTexCoord *= screenRatio;
	 
	 		deltaTexCoord.x /= screenRatio;
	 
	    // Store initial sample.
	    vec3 color = vec3(0.0);
	 
	    // Set up illumination decay factor.
	    float illuminationDecay = 0.1f;
	 
	    for (int i = 0; i < 3 ; i++)
	    {
	    
	    	vec3  core = texture2D(firstPass, ScreenLightPosition.xy  - vec2(float(i)*0.43333,float(i)*0.33333) * Param2.w).rgb;
		            
	        float bright = dot(core, vec3(0.133));
	    
		        // Step sample location along ray.
		        if(fwd)
		            texCoord -= deltaTexCoord*1.0;
		        else
		            texCoord += deltaTexCoord*1.0;
		 
			 	if(texCoord.y > 1.0 || texCoord.y < 0.0 || texCoord.x > 1.0 || texCoord.x < 0.0){
			 		continue;
			 	}
			 	
		 		vec2 flareTC = texCoord;
		 		flareTC.x *= 0.45;
		 		flareTC.x += xTC;
		 		
		 		
		 		
		        // Retrieve sample at new location.
		        vec4 sample = texture2D(Texture, flareTC);
		 
		        // Apply sample attenuation scale/decay factors.
		        sample *= illuminationDecay * weight;
		        
		 
		        // Accumulate combined color.
		        color += sample.xyz * bright ;
		 
		        // Update exponential decay factor.
		        illuminationDecay *= decay;
	    }
	 
	    return vec4(color,1);
	}
		
	
	
	vec4 addGodRay()
	{
	    vec2 flareTC = (gl_TexCoord[0].st - vec2(0.4));
	    
	    
	   // vec2 flareTC = (gl_TexCoord[0].st - vec2(0.5)) * Param1.z;
	    
	    
	    vec3 conv = vec3(0.0);
	
	     for (float x=-Param2.x; x<=Param2.x; x+=Param2.y)
	     {
	         for (float y=-Param2.x; y<=Param2.x; y+=Param2.y)
	         {
	            vec3  core = texture2D(firstPass, Param1.xy  - vec2(x,y) * Param2.w).rgb;
	            
	            float bright = dot(core, vec3(0.333));
	            if (bright > 0.89){
		            vec3 flare = texture2D(Texture,  flareTC + vec2(x,y) * Param1.w).rgb;
		            conv += flare*core;
	            }
	        }
	    }
		vec4 frag;
		frag.rbg = conv * Param2.z;
		frag.a = 0.50;

    		
    	return frag;
	}
	
	void main() {
	
	
		vec2 deltaTextCoord = vec2( gl_TexCoord[0].st - Param1.xy );
		vec2 textCoo = gl_TexCoord[0].st;
		
		deltaTextCoord *= 0.99 / float(NUM_SAMPLES) * Sdensity;
		
		float illuminationDecay = 0.1;
		
		vec4 fragCol = vec4(0.0,0.0,0.0,0.0);
		for(int i = 0; i < NUM_SAMPLES; i++) {
		
			textCoo -= deltaTextCoord;
			vec4 sample = texture2D(firstPass, textCoo );
			
			//sample *= illuminationDecay * Sweight;
			sample *= Sweight;
			fragCol += sample;
			illuminationDecay *= Sdecay;
			
		}
		fragCol *= tint*0.25;
		
		vec2 coord = gl_TexCoord[0].st;
		
		
		vec4 lf = DoLenseFlare(	Param1.xy,	coord,		true, 	weight, 	0.0,  3.0);
		lf += DoLenseFlare(		Param1.xy,	coord,		false, 	weight2, 	0.75, 4.0);
		lf += DoLenseFlare(		Param1.xy,	coord,		true, 	weight3, 	0.0, 11.0);
		lf += DoLenseFlare(		Param1.xy,	coord,		false, 	weight4, 	0.25, 12.0);
		
		vec4 col = ( lf * exposure) * 0.9;
		
		vec4 dirt = col*texture2D(Dirt, -gl_TexCoord[0].st).rrrr*0.4;
		
		fragCol += col;
		
		gl_FragColor = (fragCol+dirt); 
	}
